import processing.core.*; 
import processing.xml.*; 

import com.jsyn.engine.SynthesisEngine; 
import com.jsyn.unitgen.LineOut; 
import com.jsyn.unitgen.SineOscillator; 

import java.applet.*; 
import java.awt.Dimension; 
import java.awt.Frame; 
import java.awt.event.MouseEvent; 
import java.awt.event.KeyEvent; 
import java.awt.event.FocusEvent; 
import java.awt.Image; 
import java.io.*; 
import java.net.*; 
import java.text.*; 
import java.util.*; 
import java.util.zip.*; 
import java.util.regex.*; 

public class DopplerShift extends PApplet {

/*

 A little applet demonstrating Doppler shift
 
 Douglas Repetto, Oct 2011
 
 NOTE: the pitches played are only there to 
 help you keep track of when the wavefronts
 arrive at each point. They do not represent
 the frequency of the source! The actual
 audio frequency of this source would be
 very low, well below audio range. The thing
 to pay attention to is not the pitch of the 
 bleeps themselves, but the timing of their
 arrival at each point.
 
 */





SynthesisEngine synth;
SineOscillator leftOsc;
SineOscillator rightOsc;
LineOut lineOut;


int w = 1000;
int h = 500;
int midW = w/2;
int midH = h/2;

float sourceX = midW;
float sourceY = midH;

float sourceShiftX = 0.0f;
float sourceShiftY = 0.0f;


//background color
int bgC = color(0, 0, 0);
//sound source color
int sC = color(255, 255, 255);
//wave color
int wC = color(0, 127, 127);
//text color
int tC = color(255, 0, 0);

int step = 0;
//steps per cycle
double sPC = 40;


//wave front number
int wFN = 0;

WaveFront waveFronts[] = new WaveFront[100];


public void setup()
{
  size(w, h);
  //frameRate(20);
  smooth();

  //Create a context for the synthesizer.
  synth = new SynthesisEngine();
  //Add a tone generator.
  synth.add( leftOsc = new SineOscillator() );
  synth.add( rightOsc = new SineOscillator() );
  //Add an output mixer.
  synth.add( lineOut = new LineOut() );
  //Connect the oscillator to the output.
  leftOsc.output.connect( 0, lineOut.input, 0 );
  rightOsc.output.connect( 0, lineOut.input, 1 );

  leftOsc.frequency.set(0.0f);
  rightOsc.frequency.set(0.0f);

  // Start synthesizer using default stereo output at 44100 Hz.
  synth.start();
  //We only need to start the LineOut. It will pull data from the oscillator.
  lineOut.start();

}


public void draw()
{
  if (step % sPC == 0)
  {
    //create a wave front
    waveFronts[wFN] = new WaveFront(PApplet.parseInt(sourceX), PApplet.parseInt(sourceY), 10, 1);
    wFN++;

    if (wFN == 100)
      wFN = 0;
  }

  background(bgC);

  fill(tC);
  text("click mouse to move source", 10, h - 20);

  stroke(sC);
  fill(sC);
  ellipse(PApplet.parseInt(sourceX), PApplet.parseInt(sourceY), 10, 10);
  ellipse(0, midH, 10, 10);
  ellipse(w - 1, midH, 10, 10);

  noFill();
  stroke(wC);

  for (int i = 0; i < waveFronts.length; i++)
  {
    if (waveFronts[i] != null)
    {
      WaveFront wF = waveFronts[i];

      int x = PApplet.parseInt(wF.wFX);
      int y = PApplet.parseInt(wF.wFY);
      int d = PApplet.parseInt(wF.wFD);

      ellipse(x, y, d, d);

      wF.step();
    }
  }

  //check to see if it's time to bleep!
  int c = get(w - 1, midH);
  if (c != sC)
  {
    // println("r: " + red(c) + " g: " + green(c) + " b: " + blue(c));
    rightOsc.frequency.set(300.0f);
  }
  else
    rightOsc.frequency.set(0.0f);

  //check to see if it's time to bleep!
  c = get(0, midH);
  if (c != sC)
  {
    // println("r: " + red(c) + " g: " + green(c) + " b: " + blue(c));
    leftOsc.frequency.set(400.0f);
  }
  else
    leftOsc.frequency.set(0.0f);

  if (step % 3 == 0)
  {
    sourceX += sourceShiftX;

    if (sourceX >= w || sourceX <= 0.0f)
      sourceShiftX *= -1.0f;

    sourceY += sourceShiftY;

    if (sourceY >= h || sourceY <= 0.0f)
      sourceShiftY *= -1.0f;
  }
  step++;
}

public void mousePressed()
{
  int x = mouseX;
  int y = mouseY;


  float xDiff = mouseX - sourceX;
  float yDiff = mouseY - sourceY;

  float distance = sqrt((xDiff * xDiff) + (yDiff * yDiff));

  println("distance: " + distance);

  sourceShiftX = xDiff/distance;
  sourceShiftY = yDiff/distance;
  println("new x: " + sourceShiftX + " new y: " + sourceShiftY);
}



class WaveFront
{

  float wFD = 0.0f;
  float wFX = 0.0f;
  float wFY = 0.0f;

  float pixPerStepD = 0.0f;

  WaveFront(int x, int y, int d, float pPSD)
  {
    wFX = x;
    wFY = y;
    wFD = d;
    
    pixPerStepD = pPSD;
  }
  
  public void step()
  {
    wFD += pixPerStepD;
  }
}

  static public void main(String args[]) {
    PApplet.main(new String[] { "--bgcolor=#FFFFFF", "DopplerShift" });
  }
}
